@@ -28,6 +28,7 @@
 #include <dirent.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <time.h>
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
@@ -57,6 +58,10 @@
   #include "xdgbase.h"
 #endif
 
+#ifdef __APPLE__
+#include <mach/mach_time.h>
+#endif
+
 char *program_name;
 
 long long minsize = -1;
@@ -140,6 +145,38 @@ char *fmttime(time_t t) {
   return buf;
 }
 
+// current timestamp in milliseconds
+uint64_t now64(void)
+{
+#ifdef __APPLE__
+    uint64_t absolute = mach_absolute_time() / (1000 * 1000);
+
+    static mach_timebase_info_data_t sTimebaseInfo;
+    if ( sTimebaseInfo.denom == 0 ) {
+        mach_timebase_info(&sTimebaseInfo);
+    }
+
+    if ( sTimebaseInfo.numer == 1 && sTimebaseInfo.denom == 1 ) {
+        return absolute;
+    }
+
+    return absolute * sTimebaseInfo.numer / sTimebaseInfo.denom;
+#elif defined(_WIN32)
+    /* Warning (from documentation)
+
+    The resolution of the GetTickCount64 function is limited to
+    the resolution of the system timer, which is typically in
+    the range of 10 milliseconds to 16 milliseconds
+    */
+    return (uint64_t)GetTickCount64();
+#else
+    struct timespec timecheck;
+
+    clock_gettime(CLOCK_MONOTONIC, &timecheck);
+    return (uint64_t)timecheck.tv_sec * 1000 + (uint64_t)timecheck.tv_nsec / (1000 * 1000);
+#endif
+}
+
 char **cloneargs(int argc, char **argv)
 {
   int x;
@@ -267,6 +304,8 @@ int grokdir(char *dir, file_t **filelistp, struct stat *logfile_status)
   struct stat linfo;
   static int progress = 0;
   static char indicator[] = "-\\|/";
+  static uint64_t last_progress = 0;
+  static uint64_t now = 0;
   char *fullname, *name;
   char *fullpath = 0;
 #ifndef NO_SQLITE
@@ -302,8 +341,12 @@ int grokdir(char *dir, file_t **filelistp, struct stat *logfile_status)
 
     if (strcmp(dirinfo->d_name, ".") && strcmp(dirinfo->d_name, "..")) {
       if (!ISFLAG(flags, F_HIDEPROGRESS)) {
-	fprintf(stderr, "\rBuilding file list %c ", indicator[progress]);
-	progress = (progress + 1) % 4;
+        now = now64();
+        if ( now - last_progress > FDUPES_PROGRESS_REFRESH_MS ) {
+          fprintf(stderr, "\rBuilding file list %c ", indicator[progress % 4]);
+          last_progress = now;
+          progress++;
+        }
       }
 
       newfile = (file_t*) malloc(sizeof(file_t));
@@ -803,12 +846,11 @@ void summarizematches(file_t *files)
   else
   {
     if (numbytes < 1024.0)
-      printf("%d duplicate files (in %d sets), occupying %.0f bytes.\n\n", numfiles, numsets, numbytes);
+      printf("%s%d duplicate files (in %d sets), occupying %.0f bytes.\n\n", ISFLAG(flags, F_QUICKSUMMARY) ? "approximately " : "", numfiles, numsets, numbytes);
     else if (numbytes <= (1000.0 * 1000.0))
-      printf("%d duplicate files (in %d sets), occupying %.1f kilobytes\n\n", numfiles, numsets, numbytes / 1000.0);
+      printf("%s%d duplicate files (in %d sets), occupying %.1f kilobytes\n\n", ISFLAG(flags, F_QUICKSUMMARY) ? "approximately " : "", numfiles, numsets, numbytes / 1000.0);
     else
-      printf("%d duplicate files (in %d sets), occupying %.1f megabytes\n\n", numfiles, numsets, numbytes / (1000.0 * 1000.0));
- 
+      printf("%s%d duplicate files (in %d sets), occupying %.1f megabytes\n\n", ISFLAG(flags, F_QUICKSUMMARY) ? "approximately " : "", numfiles, numsets, numbytes / (1000.0 * 1000.0));
   }
 }
 
@@ -1394,6 +1436,8 @@ void help_text()
   printf(" -S --size               show size of duplicate files\n");
   printf(" -t --time               show modification time of duplicate files\n");
   printf(" -m --summarize          summarize dupe information\n");
+  printf(" -M --quicksummary       summarize dupe information quickly, skipping the\n");
+  printf("                         slower byte-for-byte match confirmation\n");
   printf(" -q --quiet              hide progress indicator\n");
   printf(" -d --delete             prompt user for files to preserve and delete all\n");
   printf("                         others; important: under particular circumstances,\n");
@@ -1463,6 +1507,8 @@ int main(int argc, char **argv) {
   filetree_t *checktree = NULL;
   int filecount = 0;
   int progress = 0;
+  uint64_t last_progress = 0;
+  uint64_t now = 0;
   char **oldargv;
   int firstrecurse;
   int foundoption;
@@ -1496,6 +1542,7 @@ int main(int argc, char **argv) {
     { "noprompt", 0, 0, 'N' },
     { "immediate", 0, 0, 'I'},
     { "summarize", 0, 0, 'm'},
+    { "quicksummary", 0, 0, 'M'},
     { "summary", 0, 0, 'm' },
     { "permissions", 0, 0, 'p' },
     { "order", 1, 0, 'o' },
@@ -1516,7 +1563,7 @@ int main(int argc, char **argv) {
 
   oldargv = cloneargs(argc, argv);
 
-  while ((opt = GETOPT(argc, argv, "frRq1StsHG:L:nAdPvhNImpo:il:Dcx:"
+  while ((opt = GETOPT(argc, argv, "frRq1StsHG:L:nAdPvhNImMpo:il:Dcx:"
 #ifdef HAVE_GETOPT_H
           , long_options, NULL
 #endif
@@ -1592,6 +1639,10 @@ int main(int argc, char **argv) {
     case 'm':
       SETFLAG(flags, F_SUMMARIZEMATCHES);
       break;
+    case 'M':
+      SETFLAG(flags, F_SUMMARIZEMATCHES);
+      SETFLAG(flags, F_QUICKSUMMARY);
+      break;
     case 'p':
       SETFLAG(flags, F_PERMISSIONS);
       break;
@@ -1842,7 +1893,7 @@ int main(int argc, char **argv) {
               ordertype == ORDER_CTIME ? sort_pairs_by_ctime :
                                          sort_pairs_by_filename, loginfo );
       }
-      else if (ISFLAG(flags, F_DEFERCONFIRMATION) || confirmmatch(file1, file2))
+      else if (ISFLAG(flags, F_DEFERCONFIRMATION) || ISFLAG(flags, F_QUICKSUMMARY) || confirmmatch(file1, file2))
         registerpair(match, curfile,
             ordertype == ORDER_MTIME ? sort_pairs_by_mtime :
             ordertype == ORDER_CTIME ? sort_pairs_by_ctime :
@@ -1855,8 +1906,12 @@ int main(int argc, char **argv) {
     curfile = curfile->next;
 
     if (!ISFLAG(flags, F_HIDEPROGRESS)) {
-      fprintf(stderr, "\rProgress [%d/%d] %d%% ", progress, filecount,
-       (int)((float) progress / (float) filecount * 100.0));
+      now = now64();
+      if ( now - last_progress > FDUPES_PROGRESS_REFRESH_MS ) {
+        last_progress = now;
+        fprintf(stderr, "\rProgress [%d/%d] %d%% ", progress, filecount,
+         (int)((float) progress / (float) filecount * 100.0));
+      }
       progress++;
     }
   }
